from imports.src.abstract_utilities.file_utils.src.file_filters import *

file_path = "/home/flerb/Pictures/gts/gts2/73w"
sysroot="/home/flerb/Downloads"

###get_dot_fro_line(line,dirname=None,file_path=None,get_info=False)

defaults = define_defaults()
input(defaults)
